<?php
// Quick-LP
// Copyright (c) Radius17
// Web: https://quick-lp.ru
// Commercial license https://quick-lp.ru/license.txt
// THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
// Revision: f39ef0b (2020-08-03 10:03:18)
//

defined('_QUICK_LP') or die("Access denied");

define('DE_MESSAGE','message');
define('DE_WARNING','warning');
define('DE_ERROR','error');
define('DE_MILESTONE','milestone');

class QLPDebugger {
	//---------- Singleton implementation ------------
	private static $_instance = null;

	public static function createInstance() {
		if (self::$_instance == null) {
			self::$_instance = new self();
			self::$_instance->_enabled = QLPConfig::$debugMode;
		}
	}

	public static function getInstance() {
		self::createInstance();
		return self::$_instance;
	}
	//------------------------------------------------

	private $_enabled 		= false;
	private $_events 		= array();
	private $_sqlQueries	= array();
	private $_translations	= array();
	private $_startTime		= 0;
	private $_lastTime		= 0;
	private $_lastMemory	= 0;
	private $_threshold		= 5; // в процентах
	private $_mem_threshold = 250000 ; // в байтах
	private function __construct() {
		$this->_startTime = microtime(1);
		$this->_lastTime = microtime(1);
	}

	private function passEvent($type,$text) {
		if($this->_enabled){
			$event = new stdClass();
			if($type==DE_MILESTONE){
				$newtime=microtime(1);
				$event->time = $newtime-$this->_lastTime;
				$event->ttime = $newtime-$this->_startTime;
				$this->_lastTime=$newtime;
				$newmem=$this->getMemory();
				$event->memory = $newmem-$this->_lastMemory;
				$this->_lastMemory=$newmem;
			} else {
				$event->time = 0; $event->memory = 0;
			}
			$event->type = $type;
			$event->text = $text;

			$this->_events []= $event;
		}
	}
	public function getTime() {
		$execTime = microtime(1) - $this->_startTime;
		return $execTime;
	}
	public function message($text) {
		$this->passEvent(DE_MESSAGE,$text);
	}
	public function warning($text) {
		$this->passEvent(DE_WARNING,$text);
	}
	public function error($text,$isFatal=false, $codeerror="503") {
		$this->passEvent(DE_ERROR,$text);
		if ($isFatal) QLPUtil::fatalError($text, $codeerror);
	}
	public function milestone($text) {
		$this->passEvent(DE_MILESTONE,$text);
	}
	public function translation($txt) {
		if($this->_enabled) $this->_translations []= $txt;
	}
	public function sqlQuery($query,$prefix="") {
		if($this->_enabled)	$this->_sqlQueries[]= $prefix.htmlspecialchars($query);
	}
	public function dumpSystemInfo() {
		$this->message(Text::_("Memory limit")." = ".ini_get("memory_limit"));
	}
	public function dump($direct=true) {
		$dbg_div="";
		if ($this->_enabled==2 || ($this->_enabled==1 && QLPSession::getInstance()->isAdmin())) {
			$dbg_div .= "<div id=\"debugger\">\n";
			// Debug log
			$dbg_div .= "<p><b>Debug messages:</b><p>\n";
			$execTime = microtime(1) - $this->_startTime;
			
			$dbg_div .= "<p>--Common execution time:&nbsp;".$execTime." sec.</p>";
			$dbg_div .= "<p>--Time threshold:&nbsp;".$this->_threshold." %.</p>";
			$dbg_div .= "<p>--Memory threshold: ".number_format($this->_mem_threshold,0)." bytes</tp>";
			$dbg_div .= "<p>--".$this->getMemoryString(true)." sec.</p>";
			$dbg_div .= "<p>--Memory limit = ".ini_get("memory_limit")."</p>";
			$time_threshold=($this->_threshold*$execTime)/100;
			foreach ($this->_events as $event) {
				switch ($event->type) {
					case DE_ERROR:
						$dbg_div .= "<p class=\"debug-error\">!!! Error: ".$event->text."</p>\n";
						break;

					case DE_WARNING:
						$dbg_div .= "<p class=\"debug-warning\">*** Warning: ".$event->text."</p>\n";
						break;

					case DE_MILESTONE:
						$mem=($event->memory>0 ? "+" : "-").number_format($event->memory,0," "," ");
						if ($event->memory > $this->_mem_threshold) {
							$mem = "<span class=\"red\">".$mem."</span>";
						}
						if ($event->time > $time_threshold) {
							$cur_time = "<span class=\"red\">".number_format($event->time,6)."</span>";
						} else {
							$cur_time = number_format($event->time,6);
						}
						$total_time="<span class=\"blue\">".number_format($event->ttime,6)."</span>";
						$dbg_div .= "<p>--".$event->text." => <span class=\"debug-milestone\">Time: ".$cur_time." sec, Total time ".$total_time." sec, Memory: ".$mem." bytes</span></p>\n";
						break;

					case DE_MESSAGE:
					default:
						$dbg_div .= "<p>-".$event->text."</p>\n";
						break;
				}
			}

			if(count($this->_translations)){
				// Translation log
				$_npp=0;
				$trans=array_unique($this->_translations);
				$dbg_div .= "<p id=\"debugger_trans\"><br /><b>Translation errors log (".count($trans)."):</b></p>\n";
				if (count($trans)>0) {
					$dbg_div .= "<p>";
					foreach ($trans as $trans_text) {
						$_npp++;
						$dbg_div .= $trans_text."=\"".$trans_text."\"<br />";
					}
					$dbg_div .= "</p>";
				} else { $dbg_div.="No errors found.";
				}
			}
			if(count($this->_sqlQueries)){
				// SQL log
				$dbg_div .= "<p id=\"debugger_sql\"><br /><b>SQL queries:</b><br /><br />\n";
				$_npp=0;
				foreach ($this->_sqlQueries as $sqlQuery) {
					$_npp++;
					$dbg_div .= $_npp.") ".$sqlQuery."<br />";
				}
				$dbg_div .= "</p>";
			}
			$dbg_div .= "</div>";
		}
		if($direct) echo $dbg_div;
		else return $dbg_div;
	}
	public function getMemory($peak=false)  {
		static $isWin;
		if (($peak)&& (function_exists( 'memory_get_peak_usage' ))) {
			return memory_get_peak_usage(true);
		} elseif (function_exists( 'memory_get_usage' )) {
			return memory_get_usage();
		} else {
			// Determine if a windows server
			if (is_null( $isWin )) {
				$isWin = (substr(PHP_OS, 0, 3) == 'WIN');
			}
			// Initialize variables
			$output = array();
			$pid = getmypid();
			if ($isWin) {
				// Windows workaround
				@exec( 'tasklist /FI "PID eq ' . $pid . '" /FO LIST', $output );
				if (!isset($output[5])) {
					$output[5] = null;
				}
				return $this->get_digits($output[5])*1024;
			} else {
				@exec("ps -o rss -p $pid", $output);
				return $output[1] *1024;
			}
		}
	}
	public function getMemoryString($peak=false)  {
		$pid = getmypid();
		$byte_uses=$this->getMemory($peak);
		$mb=intval($byte_uses/1048576);
		$kb=intval(($byte_uses-($mb*1048576))/1024);
		$byte=intval($byte_uses-($mb*1048576)-($kb*1024));
		return "PID - $pid .:".strval($mb).' mb. '.strval($kb).' kb. '.strval($byte). ' b.';
	}
	public static function log2file($data="", $file="log.txt") {
		QLPUtil::dump2file($data, $file);
	}
	public static function dump2file($data="", $file="log.txt") {
		QLPUtil::dump2file($data, $file);
	}
}

?>